#include <string.h>
#include <pthread.h>
#include <gst/gst.h>

#include "dvr.h"

DVR::DVR() : 
  filename(NULL),
  started(false), 
  ended(false)
{
  gst_init(0, NULL);
}

DVR::~DVR() {
  if(started) {
    stop();
  }

  gst_deinit();

  if(filename) delete []filename;
}

void DVR::setFileName(const char *_filename) {
  if(filename) delete []filename;
  filename=new char[strlen(_filename)+1];
  strcpy(filename, _filename);
}

bool DVR::start() {
  started=true;
  ended=false;
  pthread_create(&thread_gst, NULL, run, this);
  return true;
}

bool DVR::stop() {
  g_main_loop_quit(loop);
  pthread_join(thread_gst, NULL);
  started=false;
  return true;
}

bool DVR::isStarted() {
  return started;
}

bool DVR::isEnded() {
  return ended;
}

void *DVR::run(void *data) {
  DVR *me=(DVR*)data;
  me->context=g_main_context_new();
  me->loop=g_main_loop_new (NULL, FALSE);

  GstElement *pipeline,
             *video_source,
             *video_convert,
             *video_rate,
             *video_enc,
             *avimux,
             *file_output;

  GstCaps *filter;


  pipeline=gst_pipeline_new ("dvr");
  video_source=gst_element_factory_make ("v4lsrc", "video_source");
  video_rate=gst_element_factory_make ("videorate", "video_rate");
  video_convert=gst_element_factory_make("ffmpegcolorspace", "video_convert");
  video_enc=gst_element_factory_make ("ffenc_mpeg4", "video_enc");
  avimux=gst_element_factory_make("avimux", "avimux");
  file_output=gst_element_factory_make ("filesink", "file_output");

  gst_bin_add_many(GST_BIN(pipeline), video_source, video_convert, video_rate, video_enc, avimux, file_output, NULL);

  filter=gst_caps_new_simple("video/x-raw-rgb",
                             "width", G_TYPE_INT, 352,
                             "height", G_TYPE_INT, 288,
                             "framerate", GST_TYPE_FRACTION, 15, 1,
                             NULL);

  gst_element_link(video_source, video_rate);
  gst_element_link_filtered(video_rate, video_convert, filter);
  gst_element_link(video_convert, video_enc);
  gst_element_link(video_enc, avimux);
  gst_element_link(avimux, file_output);
  
  g_object_set(G_OBJECT(file_output), "location", me->filename, NULL);

  gst_element_set_state(pipeline, GST_STATE_PLAYING);
  g_main_loop_run(me->loop);
  gst_element_set_state(pipeline, GST_STATE_NULL);

  g_main_loop_unref(me->loop);
  g_main_context_unref(me->context);
  me->ended=true;

  return NULL;
}
