#include <pthread.h>
#include <sys/vfs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <qdatetime.h>
#include <qlabel.h>
#include <qprogressbar.h>

#include "tab_monitoring.h"

TabMonitoring::TabMonitoring(QWidget *parent, const char *name) : UITabMonitoring(parent, name) {}

TabMonitoring::~TabMonitoring() {}

void TabMonitoring::start(const QString &_filename) {
  filename=_filename;
  end=false;
  pthread_create(&thread_monitor, NULL, run, this);
}

void TabMonitoring::stop() {
  end=true;
  pthread_join(thread_monitor, NULL);
}

int TabMonitoring::cpuLoad() {
	static QTime old_time;
	static int old_jiffies=0;
	int user, system;

	FILE *file=fopen("/proc/stat", "r");
	if(file==NULL) return -1;
	QTime new_time=QTime::currentTime();
	fscanf(file, "%*s %d %*s %d", &user, &system);
	fclose(file);
	int new_jiffies=user+system;

	if(old_jiffies==0) {
		old_time=new_time;	old_jiffies=new_jiffies;
		return 0;
	}

	int load=(new_jiffies-old_jiffies)*1000/old_time.msecsTo(new_time);
	old_time=new_time;
	old_jiffies=new_jiffies;

	return load;
}

void *TabMonitoring::run(void *data) {
  TabMonitoring *me=(TabMonitoring*)data;

	double delta, max_duration, virtual_duration;
  double total_space, available_space, actual_file_size;
	struct timeval now, before, start;
	struct stat s;
	struct statfs sfs;
	QTime t;
	
	statfs(me->filename, &sfs);
	total_space=double(sfs.f_bsize)*double(sfs.f_blocks);

  gettimeofday(&start, NULL);

	gettimeofday(&before, NULL);
	
	do {
		sleep(1);

		gettimeofday(&now, NULL);
				
		delta=now.tv_sec-start.tv_sec+(now.tv_usec-start.tv_usec)/1000000.0;
		t=QTime();
		me->qlb_duration->setText(t.addSecs(int(delta)).toString());
	
		me->qpb_cpu_load->setProgress(me->cpuLoad());
		
		stat(me->filename, &s);

    actual_file_size=double(s.st_size);
		me->qlb_actual_file_size->setText(QString::number(actual_file_size/1048576.0, 'f', 1)+" MB");
		me->qlb_average_data_rate->setText(QString::number((actual_file_size/delta)/1024.0, 'f', 1)+" KB/s");

		statfs(me->filename, &sfs);
		available_space=double(sfs.f_bsize)*double(sfs.f_bavail);	
		me->qlb_free_space->setText(QString::number(int(available_space/1048576.0))+" MB");
		me->qpb_free_space->setProgress(int(available_space/total_space*100.0));

		max_duration=delta*total_space/actual_file_size;
		t=QTime();
		if(max_duration<86400) {
			me->qlb_max_estimated_duration->setText(t.addSecs(int(max_duration)).toString());
		} else {
			me->qlb_max_estimated_duration->setText(QString::number(int(max_duration)/86400)+trUtf8(" day(s), ")+t.addSecs(int(max_duration)).toString());
		}

		if(true /*nb_max_frames==0*/) {
			me->qlb_estimated_file_size->setText(QString::number((available_space+actual_file_size)/1048576.0, 'f', 1)+" MB");
		} else {
			me->qlb_estimated_file_size->setText(QString::number(max_duration*actual_file_size/(virtual_duration*1048576.0), 'f', 1)+" MB");
		}

		//me->qpb_duration->setProgress(int(100*virtual_duration/max_duration));
				
	} while(!me->end);

	return NULL;
}
